@@ -7,12 +7,9 @@ from Shared.DC.Scripts.Bindings import Bindings
 from Errors import ValidationError
 from Products.Formulator.Widget import MultiItemsWidget
 from helpers import is_sequence, convert_unicode
+from Products.Formulator.i18n import translate as _
 
-try:
-    from Products.PlacelessTranslationService import translate
-    have_pts = 1
-except ImportError:
-    have_pts = 0
+from Products.PageTemplates.Expressions import SecureModuleImporter
 
 class Field:
     """Base class of all fields.
@@ -37,11 +34,11 @@ class Field:
         # initialize overrides of fields in form
         self.initialize_overrides()
 
-        # initialize message values with defaults
+        # initialize message values empty
         message_values = {}
-        for message_name in self.validator.message_names:
-            message_values[message_name] = getattr(self.validator,
-                                                   message_name)
+        #for message_name in self.validator.message_names:
+        #    message_values[message_name] = getattr(self.validator,
+        #                                           message_name)
         self.message_values = message_values
 
     security.declareProtected('Change Formulator Fields', 'initialize_values')
@@ -114,6 +111,7 @@ class Field:
             if not kw.has_key('here'):
                 kw['here'] = self.aq_parent
             kw['request'] = self.REQUEST
+            kw['modules'] = SecureModuleImporter
             value = tales_expr.__of__(self)(
                 field=self,
                 form=self.aq_parent, **kw)
@@ -125,59 +123,28 @@ class Field:
             else:
                 # get normal value
                 value = self.get_orig_value(id)
-        
-        # check if an i18n id exist
-        if id in ['title', 'description', 'items'] and have_pts:
-            if id == 'items':
-                return self.i18n_translate_items(value)
-            else:
-                msgstr = self.i18n_translate(id)
-            if msgstr is not None:
-                return msgstr
-                
 
         # if normal value is a callable itself, wrap it
         if callable(value):
             return value.__of__(self)
-        else:
-            return value
-
-    def get_i18n_info(self):
-        """Get i18n info. Try to get it from the form."""
-        try:
-            return self.aq_parent.get_i18n_info()
-        except AttributeError:
-            return '', ''
     
-    def i18n_translate(self, appendstr):
-        """creates msgid and translates it
-        """
-        i18n_domain, i18n = self.get_i18n_info()
-        if i18n_domain and i18n:
-            msgid = "%s_%s_%s" % (i18n, self.id, appendstr)
-            msgstr = translate(i18n_domain, msgid, context=self.REQUEST,
-                as_unicode=self.get_unicode_mode())
-            return msgstr
-        
-        return None
+        # create message id for title and description in right domain
+        if id in ['title', 'description']:
+            i18n_domain = self.get_i18n_domain()
+            if i18n_domain:
+                return _(value, i18n_domain)
+        return value
 
-    def i18n_translate_items(self, list):
-        """ translates list keys"""
-        values = []
-        for i in range(len(list)):
-            item = list[i]
-            if is_sequence(item):
-                k, v = item
-            else:
-                k = v = item
-            msgstr = self.i18n_translate(i)
-            if msgstr is not None:
-                values.append((msgstr,v))
-            else:
-                values.append((k,v))
+    # this also works if field is not in form for testing
+    # reasons..
+    def get_i18n_domain(self):
+        try:
+            # try to acquire it
+            return self.aq_inner.aq_parent.get_i18n_domain()
+        except AttributeError:
+            # otherwise, return empty domain
+            return ''
         
-        return values
-            
     security.declareProtected('View management screens', 'get_override')
     def get_override(self, id):
         """Get override method for id (not wrapped)."""
@@ -222,20 +189,37 @@ class Field:
         return '%s.subfield_%s_%s:record' % (self.field_record, self.id, id)
 
     security.declareProtected('View management screens', 'get_error_message')
-    def get_error_message(self, name):
+    def get_error_message(self, name, want_message_id=True):
         try:
-            message = self.message_values[name]
-            if have_pts:
-                msgstr = self.i18n_translate(name)
-                if msgstr is not None:
-                   message = msgstr
-            return message
+            # look up message in field
+            result = self.message_values[name]
         except KeyError:
+            # if we can't find it in field, look it up in form
+            # these will be the correct message ids, so return result
+            # directly
             if name in self.validator.message_names:
-                return getattr(self.validator, name)
+                result = getattr(self.validator, name)
             else:
-                return "Unknown error: %s" % name
-
+                result = "Unknown error: %s" % name
+            # if we don't want message id, strip it off
+            if not want_message_id:
+                try:
+                    # looks into message id internals..
+                    result = result.ustr
+                except AttributeError:
+                    pass
+            return result
+        if want_message_id:
+            # we do want a message id, so construct one from form domain
+            result = _(result, self.get_i18n_domain())
+        else:
+            try:
+                # looks into message id internals..
+                result = result.ustr
+            except AttributeError:
+                pass
+        return result
+    
     security.declarePrivate('_render_helper')
     def _render_helper(self, key, value, REQUEST):
         value = self._get_default(key, value, REQUEST)
@@ -549,7 +533,9 @@ class ZMIField(
             message = REQUEST[message_key]
             if unicode_mode:
                 message = unicode(message, 'UTF-8')
-            messages[message_key] = message
+            # only save message if we're indeed changing from original
+            if getattr(self.validator, message_key) != message:
+                messages[message_key] = message
 
         self.message_values = messages
         if REQUEST:
