@@ -20,6 +20,7 @@ from Widget import render_tag
 from DummyField import fields
 from FormToXML import formToXML
 from XMLToForm import XMLToForm
+from helpers import convert_unicode
 
 # FIXME: manage_renameObject hack needs these imports
 from Acquisition import aq_base
@@ -37,8 +38,6 @@ class Form:
     stored_encoding = 'ISO-8859-1'
     unicode_mode = 0
     i18n_domain = ''
-    i18n = ''
-    
 
     # CONSTRUCTORS
     def __init__(self, action, method, enctype, name,
@@ -206,13 +205,7 @@ class Form:
         self.group_list = group_list
         return 1
 
-    # ACCESSORS
-    security.declareProtected('View', 'get_i18n_info')
-    def get_i18n_info(self):
-        """Get i18n information of the form
-        """
-        return self.i18n_domain, self.i18n
-     
+    # ACCESSORS     
     security.declareProtected('View', 'get_fields')
     def get_fields(self, include_disabled=0):
         """Get all fields for all groups (in the display order).
@@ -291,6 +284,12 @@ class Form:
         """
         return getattr(self, 'unicode_mode', 0)
 
+    security.declareProtected('View', 'get_i18n_domain')
+    def get_i18n_domain(self):
+        """Get i18n domain, if any.
+        """
+        return getattr(self, 'i18n_domain', '')
+    
     security.declareProtected('View', 'render')
     def render(self, dict=None, REQUEST=None):
         """Render form in a default way.
@@ -480,8 +479,6 @@ class Form:
         """change form according to xml"""
         XMLToForm(xml, self, override_encoding)
 
-    security.declareProtected('Access contents information',
-                              'management_page_charset')
     def _management_page_charset(self):
         """manage_before_header calls this to determine management
         screen encodings.
@@ -641,15 +638,10 @@ def create_settings_form():
                                     title = 'i18n domain',
                                     required=0,
                                     default="")
-    i18n = fields.StringField('i18n',
-                             title='i18n id',
-                             requierd=0,
-                             default="")
                         
-
     form.add_fields([title, row_length, name, action, method,
-                     enctype, encoding, stored_encoding, unicode_mode, i18n_domain,
-                     i18n])
+                     enctype, encoding, stored_encoding, unicode_mode,
+                     i18n_domain])
     return form
 
 class ZMIForm(ObjectManager, PropertyManager, RoleManager, Item, Form):
@@ -692,6 +684,8 @@ class ZMIForm(ObjectManager, PropertyManager, RoleManager, Item, Form):
                                                'UTF-8', 'ISO-8859-1',
                                                unicode_mode)
         self.id = id
+        if unicode_mode:
+            title = convert_unicode(title)
         self.title = title
         self.row_length = 4
 
@@ -770,6 +764,8 @@ class ZMIForm(ObjectManager, PropertyManager, RoleManager, Item, Form):
         title = string.strip(title)
         if not title:
             title = id # title is always required, use id if not provided
+        if self.get_unicode_mode():
+            title = convert_unicode(title)
         # get the field class we want to add
         field_class = FieldRegistry.get_field_class(fieldname)
         # create field instance
@@ -780,6 +776,10 @@ class ZMIForm(ObjectManager, PropertyManager, RoleManager, Item, Form):
         add_and_edit(self, id, REQUEST)
         return ''
 
+    # security declaration for the generic "add field" form,
+    # which is set as a class attribute in "initializeForm" later on
+    security.declareProtected('Change Formulator Forms', 'fieldAdd')
+
     security.declareProtected('View management screens', 'formTest')
     formTest = DTMLFile('dtml/formTest', globals())
 
