@@ -84,8 +84,15 @@ def formToXML(form, prologue=1):
 
             write('        <messages>\n')
             for message_key in field.get_error_names():
+                # get message text, don't want a MessageId as we
+                # don't want to trigger translation in serialization
+                message_text = field.get_error_message(message_key,
+                                                       want_message_id=False)
+                # we don't want unicode here
+                if isinstance(message_text, unicode):
+                    message_text = message_text.encode(form.stored_encoding)
                 write('          <message name="%s">%s</message>\n' %
-                      (escape(message_key), escape(field.get_error_message(message_key))))
+                      (escape(message_key), escape(message_text)))
             write('        </messages>\n')
             write('      </field>\n')
         write('      </fields>\n')
