@@ -7,6 +7,7 @@ from urllib import urlopen
 from urlparse import urljoin
 from Errors import ValidationError
 from helpers import is_sequence
+from Products.Formulator.i18n import translate as _
 
 try:
     from DateTime.DateTime import DateError, TimeError
@@ -41,6 +42,16 @@ class ValidatorBase:
     def validate(self, field, key, REQUEST):
         pass # override in subclass
 
+    def serializeValue(self, field, value, sax_producer):
+        """Given a field, a value and a sax_producer, this method sends
+        sax events to the sax producer that represent the XMLified value.
+        """
+        pass # override in subclass
+
+    def deserializeValue(self, field, value):
+        REQUEST = {'key': value}
+        return self.validate(field, 'key', REQUEST)
+    
     def need_validate(self, field, key, REQUEST):
         """Default behavior is always validation.
         """
@@ -89,7 +100,7 @@ class StringBaseValidator(Validator):
 
     message_names = Validator.message_names + ['required_not_found']
 
-    required_not_found = 'Input is required but no input given.'
+    required_not_found = _('Input is required but no input given.')
 
     def validate(self, field, key, REQUEST):
         value = REQUEST.get(key, "")
@@ -99,6 +110,9 @@ class StringBaseValidator(Validator):
             self.raise_error('required_not_found', field)
         return value
 
+    def serializeValue(self, field, value, producer):
+        producer.handler.characters(value)
+
 class StringValidator(StringBaseValidator):
     property_names = StringBaseValidator.property_names +\
                      ['unicode', 'max_length', 'truncate']
@@ -130,7 +144,7 @@ class StringValidator(StringBaseValidator):
     message_names = StringBaseValidator.message_names +\
                     ['too_long']
 
-    too_long = 'Too much input was given.'
+    too_long = _('Too much input was given.')
 
     def validate(self, field, key, REQUEST):
         value = StringBaseValidator.validate(self, field, key, REQUEST)
@@ -153,7 +167,7 @@ StringValidatorInstance = StringValidator()
 class EmailValidator(StringValidator):
     message_names = StringValidator.message_names + ['not_email']
 
-    not_email = 'You did not enter an email address.'
+    not_email = _('You did not enter an email address.')
 
     # This regex allows for a simple username or a username in a
     # multi-dropbox (%). The host part has to be a normal fully
@@ -198,7 +212,7 @@ class PatternValidator(StringValidator):
     message_names = StringValidator.message_names +\
                     ['pattern_not_matched']
 
-    pattern_not_matched = "The entered value did not match the pattern."
+    pattern_not_matched = _("The entered value did not match the pattern.")
 
     def validate(self, field, key, REQUEST):
         value = StringValidator.validate(self, field, key, REQUEST)
@@ -216,6 +230,18 @@ class BooleanValidator(Validator):
     def validate(self, field, key, REQUEST):
         return not not REQUEST.get(key, 0)
 
+    def serializeValue(self, field, value, producer):
+        if value:
+            value_string = 'True'
+        else:
+            value_string = 'False'
+        producer.handler.characters(value_string)
+
+    def deserializeValue(self, field, value):
+        if value == 'True':
+            return True
+        return False
+        
 BooleanValidatorInstance = BooleanValidator()
 
 class IntegerValidator(StringBaseValidator):
@@ -241,8 +267,8 @@ class IntegerValidator(StringBaseValidator):
     message_names = StringBaseValidator.message_names +\
                     ['not_integer', 'integer_out_of_range']
 
-    not_integer = 'You did not enter an integer.'
-    integer_out_of_range = 'The integer you entered was out of range.'
+    not_integer = _('You did not enter an integer.')
+    integer_out_of_range = _('The integer you entered was out of range.')
 
     def validate(self, field, key, REQUEST):
         value = StringBaseValidator.validate(self, field, key, REQUEST)
@@ -263,12 +289,16 @@ class IntegerValidator(StringBaseValidator):
             self.raise_error('integer_out_of_range', field)
         return value
 
+    def serializeValue(self, field, value, producer):
+        value_string = str(value)
+        producer.handler.characters(value_string)
+    
 IntegerValidatorInstance = IntegerValidator()
 
 class FloatValidator(StringBaseValidator):
     message_names = StringBaseValidator.message_names + ['not_float']
 
-    not_float = "You did not enter a floating point number."
+    not_float = _("You did not enter a floating point number.")
 
     def validate(self, field, key, REQUEST):
         value = StringBaseValidator.validate(self, field, key, REQUEST)
@@ -281,6 +311,10 @@ class FloatValidator(StringBaseValidator):
             self.raise_error('not_float', field)
         return value
 
+    def serializeValue(self, field, value, producer):
+        value_string = str(value)
+        producer.handler.characters(value_string)
+    
 FloatValidatorInstance = FloatValidator()
 
 class LinesValidator(StringBaseValidator):
@@ -321,9 +355,9 @@ class LinesValidator(StringBaseValidator):
     message_names = StringBaseValidator.message_names +\
                     ['too_many_lines', 'line_too_long', 'too_long']
 
-    too_many_lines = 'You entered too many lines.'
-    line_too_long = 'A line was too long.'
-    too_long = 'You entered too many characters.'
+    too_many_lines = _('You entered too many lines.')
+    line_too_long = _('A line was too long.')
+    too_long = _('You entered too many characters.')
 
     def validate(self, field, key, REQUEST):
         value = StringBaseValidator.validate(self, field, key, REQUEST)
@@ -359,6 +393,10 @@ class LinesValidator(StringBaseValidator):
 
         return result
 
+    def serializeValue(self, field, value, producer):
+        value_string = '\n'.join(value)
+        producer.handler.characters(value_string)
+    
 LinesValidatorInstance = LinesValidator()
 
 class TextValidator(LinesValidator):
@@ -371,6 +409,9 @@ class TextValidator(LinesValidator):
         # join everything into string again with \n and return
         return "\n".join(value)
 
+    def serializeValue(self, field, value, producer):
+        producer.handler.characters(value)
+    
 TextValidatorInstance = TextValidator()
 
 class SelectionValidator(StringBaseValidator):
@@ -388,7 +429,7 @@ class SelectionValidator(StringBaseValidator):
     message_names = StringBaseValidator.message_names +\
                     ['unknown_selection']
 
-    unknown_selection = 'You selected an item that was not in the list.'
+    unknown_selection = _('You selected an item that was not in the list.')
 
     def validate(self, field, key, REQUEST):
         value = StringBaseValidator.validate(self, field, key, REQUEST)
@@ -443,8 +484,8 @@ class MultiSelectionValidator(Validator):
     message_names = Validator.message_names + ['required_not_found',
                                                'unknown_selection']
 
-    required_not_found = 'Input is required but no input given.'
-    unknown_selection = 'You selected an item that was not in the list.'
+    required_not_found = _('Input is required but no input given.')
+    unknown_selection = _('You selected an item that was not in the list.')
 
     def validate(self, field, key, REQUEST):
         values = REQUEST.get(key, [])
@@ -492,6 +533,13 @@ class MultiSelectionValidator(Validator):
         # everything checks out
         return result
 
+    def serializeValue(self, field, values, producer):
+        for value in values:
+            # XXX How should I handle integer types here?
+            producer.startElement('value')
+            producer.handler.characters(value)
+            producer.endElement('value')
+    
 MultiSelectionValidatorInstance = MultiSelectionValidator()
 
 class FileValidator(Validator):
@@ -548,7 +596,7 @@ class LinkValidator(StringValidator):
 
     message_names = StringValidator.message_names + ['not_link']
 
-    not_link = 'The specified link is broken.'
+    not_link = _('The specified link is broken.')
 
     def validate(self, field, key, REQUEST):
         value = StringValidator.validate(self, field, key, REQUEST)
@@ -631,9 +679,9 @@ class DateTimeValidator(Validator):
                                                'not_datetime',
                                                'datetime_out_of_range']
 
-    required_not_found = 'Input is required but no input given.'
-    not_datetime = 'You did not enter a valid date and time.'
-    datetime_out_of_range = 'The date and time you entered were out of range.'
+    required_not_found = _('Input is required but no input given.')
+    not_datetime = _('You did not enter a valid date and time.')
+    datetime_out_of_range = _('The date and time you entered were out of range.')
 
     def validate(self, field, key, REQUEST):
         try:
@@ -706,12 +754,27 @@ class DateTimeValidator(Validator):
 
         return result
 
+    def serializeValue(self, field, value, producer):
+        if value is not None:
+            value_string = value.HTML4()
+            producer.handler.characters(value_string)
+    
+    def deserializeValue(self, field, value):
+        return DateTime(value)
+    
 DateTimeValidatorInstance = DateTimeValidator()
 
-
 class SuppressValidator(ValidatorBase):
     """A validator that is actually not used.
     """
+    property_names = ValidatorBase.property_names + ['external_validator']
+    external_validator = fields.MethodField('external_validator',
+                                            title="External Validator",
+                                            description=(
+        "Ignored, as a validator isn't used here."),
+                                            default="",
+                                            required=0,
+                                            enabled=0)
     def need_validate(self, field, key, REQUEST):
         """Don't ever validate; suppress result in output.
         """
