@@ -1,9 +1,18 @@
+import os, sys
+if __name__ == '__main__':
+    execfile(os.path.join(sys.path[0], 'framework.py'))
+
+from Testing import ZopeTestCase
+
+ZopeTestCase.installProduct('Formulator')
+
 import unittest
-import ZODB
-import OFS.Application
+import xml.sax.handler
 from ZPublisher.TaintedString import TaintedString
 from Products.Formulator import Validator
 from Products.Formulator.StandardFields import DateTimeField
+from Products.Formulator.StandardFields import LinesField
+from DateTime import DateTime
 
 class TestField:
     def __init__(self, id, **kw):
@@ -21,7 +30,36 @@ class TestField:
         # XXX fake ... what if installed python does not support utf-8?
         return "utf-8"
 
-class ValidatorTestCase(unittest.TestCase):
+class FakeSaxHandler:
+    def __init__(self):
+        self._xml = ''
+
+    def startElement(self, key):
+        self._xml = self._xml + '<%s>' % key
+        
+    def endElement(self, key):
+        self._xml = self._xml + '</%s>' % key
+
+    def characters(self, characters):
+        self._xml = self._xml + characters
+        
+    def getXml(self):
+        return self._xml
+        
+class FakeSaxProducer:
+    def __init__(self):
+        self.handler = FakeSaxHandler()
+
+    def startElement(self, key):
+        self.handler.startElement(key)
+
+    def endElement(self, key):
+        self.handler.endElement(key)
+
+    def getXml(self):
+        return self.handler.getXml()
+        
+class ValidatorTestCase(ZopeTestCase.ZopeTestCase):
     def assertValidatorRaises(self, exception, error_key, f, *args, **kw):
         try:
             apply(f, args, kw)
@@ -127,8 +165,22 @@ class StringValidatorTestCase(ValidatorTestCase):
             TestField('f', max_length=0, truncate=0, required=0, unicode=0),
             'f', {'f' : TaintedString('<foo>')})
         self.assertEqual('<foo>', result)
+    
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        string = 'This is the string value'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.v.serializeValue(field, string, handler)
+        self.assertEquals('This is the string value', handler.getXml())
 
-
+    def test_deserializeValue(self):
+        string = 'This is the string value'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.assertEquals(
+            'This is the string value',
+            self.v.deserializeValue(field, string)
+            )
+        
 class LinesValidatorTestVase(ValidatorTestCase):
 
     def setUp(self):
@@ -179,6 +231,21 @@ class LinesValidatorTestVase(ValidatorTestCase):
                       truncate=0, required=1, unicode=0),
             'f', {'f': '\nToo long \ntext'} )
 
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        value = ['Two Lines ',' of Text']
+        field = TestField('f', max_lenght=0, truncate=0, required=1, unicode=0)
+        self.v.serializeValue(field, value, handler)
+        self.assertEqual('Two Lines \n of Text', handler.getXml())
+
+    def test_deserializeValue(self):
+        string = 'Two Lines \n of Text'
+        field = TestField('f', max_length=0, truncate=0, whitespace_preserve=1, required=0, unicode=1)
+        self.assertEquals(
+            ['Two Lines ', ' of Text'], 
+            self.v.deserializeValue(field, string)
+            )
+        
 class EmailValidatorTestCase(ValidatorTestCase):
 
     def setUp(self):
@@ -215,7 +282,38 @@ class EmailValidatorTestCase(ValidatorTestCase):
             TestField('f', max_length=0, truncate=0, required=1, unicode=0),
             'f', {'f': ''})
 
-# skip PatternValidator for now
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        string = 'eric@infrae.com'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.v.serializeValue(TestField, string, handler)
+        self.assertEqual('eric@infrae.com', handler.getXml())
+
+    def test_deserializeValue(self):
+        string = 'eric@infrae.com'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.assertEquals(
+            'eric@infrae.com', 
+            self.v.deserializeValue(field, string)
+            )
+        
+# not much for PatternValidator for now
+class PatternValidatorTestCase(ValidatorTestCase):
+    def setUp(self):
+        self.v = Validator.PatternValidatorInstance
+
+    def test_ff_pattern(self):
+        # test bug where an 'd','e' or 'f' in the input
+        # caused garbled output for some patterns
+        pattern='f-f'
+        value='d-1'
+        
+        field= \
+            TestField('f', max_length=0, truncate=0, required=1, unicode=0,
+                      pattern=pattern)
+        result = self.v.validate(field, 'f', {'f': value} )
+        self.assertEquals(value, result)
+
 
 class BooleanValidatorTestCase(ValidatorTestCase):
     def setUp(self):
@@ -242,6 +340,30 @@ class BooleanValidatorTestCase(ValidatorTestCase):
             'f', {})
         self.assertEquals(0, result)
 
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        value = False
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.v.serializeValue(field, value, handler)
+        self.assertEqual('False', handler.getXml())
+        handler2 = FakeSaxProducer()
+        value = True
+        self.v.serializeValue(field, value, handler2)
+        self.assertEqual('True', handler2.getXml())
+
+    def test_deserializeValue(self):
+        string = 'False'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.assertEquals(
+            False, 
+            self.v.deserializeValue(field, string)
+            )
+        string = 'True'
+        self.assertEquals(
+            True, 
+            self.v.deserializeValue(field, string)
+            )
+        
 class IntegerValidatorTestCase(ValidatorTestCase):
     def setUp(self):
         self.v = Validator.IntegerValidatorInstance
@@ -366,6 +488,21 @@ class IntegerValidatorTestCase(ValidatorTestCase):
                       start="", end=""),
             'f', {})
 
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        value = 1337
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.v.serializeValue(field, value, handler)
+        self.assertEqual('1337', handler.getXml())
+
+    def test_deserializeValue(self):
+        string = '1337'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1, start=0, end=2000)
+        self.assertEquals(
+            1337, 
+            self.v.deserializeValue(field, string)
+            )
+        
 class FloatValidatorTestCase(ValidatorTestCase):
     def setUp(self):
         self.v = Validator.FloatValidatorInstance
@@ -396,6 +533,21 @@ class FloatValidatorTestCase(ValidatorTestCase):
            TestField('f', max_length=0, truncate=0, required=1),
            'f', {'f': '1f'})
 
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        value = 1.00001
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.v.serializeValue(field, value, handler)
+        self.assertEqual('1.00001', handler.getXml())
+
+    def test_deserializeValue(self):
+        string = '1.00001'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1, start=0, end=2000)
+        self.assertEquals(
+            1.00001, 
+            self.v.deserializeValue(field, string)
+            )
+        
 class DateTimeValidatorTestCase(ValidatorTestCase):
     def setUp(self):
         self.v = Validator.DateTimeValidatorInstance
@@ -530,12 +682,35 @@ class DateTimeValidatorTestCase(ValidatorTestCase):
                   'subfield_f_hour': '10',
                   'subfield_f_minute': '61'})
 
+    def test_serializeValue(self):
+        handler = FakeSaxProducer()
+        value = self.v.validate(
+            DateTimeField('f', allow_empty_time=1),
+            'f', {'subfield_f_year': '2002',
+                  'subfield_f_month': '12',
+                  'subfield_f_day': '1',
+                  'subfield_f_hour': '10',
+                  'subfield_f_minute': '30'})
+        field = DateTimeField('f')
+        self.v.serializeValue(field, value, handler)
+        date = DateTime(2002,12,01,10,30,00)
+        self.assertEqual(date.HTML4(), handler.getXml())
+
+    def test_deserializeValue(self):
+        string = '2004-04-23T16:13:40Z'
+        field = TestField('f', max_length=0, truncate=0, required=0, unicode=1)
+        self.assertEquals(
+            DateTime('2004-04-23T16:13:40Z'), 
+            self.v.deserializeValue(field, string)
+            )
+        
 def test_suite():
     suite = unittest.TestSuite()
 
     suite.addTest(unittest.makeSuite(StringValidatorTestCase, 'test'))
     suite.addTest(unittest.makeSuite(LinesValidatorTestVase, 'test'))
     suite.addTest(unittest.makeSuite(EmailValidatorTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(PatternValidatorTestCase, 'test'))
     suite.addTest(unittest.makeSuite(BooleanValidatorTestCase, 'test'))
     suite.addTest(unittest.makeSuite(IntegerValidatorTestCase, 'test'))
     suite.addTest(unittest.makeSuite(FloatValidatorTestCase, 'test'))
@@ -543,9 +718,5 @@ def test_suite():
 
     return suite
 
-def main():
-    unittest.TextTestRunner().run(test_suite())
-
 if __name__ == '__main__':
-    main()
-
+    framework()
